#!/usr/bin/env python
# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Parseur DOM pour fichiers XML issus de AAF
#
###########################################################################

import xml.dom.minidom
import sys
                                                                                                                                                            
                                                                                                                                                            
def get_text_node(node):
    """
    @param node: node minidom contenant du texte
    Utilitaire minidom permettant de récupérer le texte d'un node texte
    """
    texte = ""
    nodelist = node.childNodes
    for textnode in nodelist:
        if textnode.nodeType == textnode.TEXT_NODE:
            texte = textnode.data
    return texte

def open_file(filename, mode):
    try:
        return file(filename, mode)
    except:
        print "Impossible d'ouvrir le fichier %s"%filename
        sys.exit()

def reverse_date(date):
    """jj/mm/aaaa -> aaaammjj"""
    date = date.replace('/','')
    if len(date) != 8:
        return '00000000'
    else:
        return '%s%s%s'%(date[4:8],date[2:4],date[0:2])

def civilite(label):
    if label == 'Mlle':
        return '3'
    elif label == 'Mme':
        return '2'
    else:
        return '1'

def ok_groupe(groupe, gtype='g'):
    """
    portage de la fonction PHP originale
    """
    if gtype == 'n':
        # niveau coupé au 1er espace
        groupe = groupe.split()[0]
    groupe = groupe.split('(')[0]     #parentheses
    groupe = groupe.replace(' ', '')  #espaces
    groupe = groupe.replace('*', 'e') #etoile, ex : mp*
    groupe = groupe.replace('.', '')  #point
    groupe = groupe.replace(':', '')  #2points
    groupe = groupe.replace('=', '')  #égal
    groupe = groupe.replace('"', '')  #double-quotes
    # groupes 100% numériques
    if groupe.isdigit():
        # ajout du prefixe "type"
        groupe = gtype+groupe
    if len(groupe) > 19:
        groupe = groupe[0:20]
    return groupe

def parse_xml_file(filename):
    """
    parsing du fichier XML AAF
    """
    global valide, f_scribe
    document = xml.dom.minidom.parse(filename)
    root_node = document.documentElement
    for request in root_node.getElementsByTagName('addRequest'):
        for operational in request.getElementsByTagName('operationalAttributes'):
            attr = operational.getElementsByTagName('attr')[0]
            #print attr.getAttribute("name"), ":", get_text_node(attr.getElementsByTagName('value')[0]), '<br>'
            if get_text_node(attr.getElementsByTagName('value')[0]) != 'PersEducNat':
                continue
        # prof : doublon avec ENTPersonJointure
        #for identifier in request.getElementsByTagName('identifier'):
        #    pass
        #    print "identifiant :", get_text_node(identifier.getElementsByTagName('id')[0]), '<br>'
        dico = {}
        for attributes in request.getElementsByTagName('attributes'):
            for attr in attributes.getElementsByTagName('attr'):
                value = ''
                for val in attr.getElementsByTagName('value'):
                    if value != '':
                        # FIXME : séparateur (1) 
                        value += '\n'
                    value += get_text_node(val)
                if value != '':
                    # FIXME : séparateur(2)
                    #value.replace('$', ' ')
                    # encode pour affichage uniquement
                    #print attr.getAttribute("name"), ":", value.encode('iso-8859-1'), '<br>'
                    dico[attr.getAttribute("name")] = value.encode('utf-8')
                else:
                    # attribut vide
                    pass
        try:
            if dico.has_key('ENTPersonDateNaissance'):
                dico['date'] = reverse_date(dico['ENTPersonDateNaissance'])
            else:
                dico['date'] += '00000000'
            if dico.has_key('personalTitle'):
                dico['sexe'] = civilite(dico['personalTitle'])
            else:
                dico['sexe'] = 0
            # classes enseignées
            dico['classes'] = ''
            if dico.has_key('ENTAuxEnsClasses'):
                for cl in dico['ENTAuxEnsClasses'].split('\n'):
                    dico['classes'] += ok_groupe(cl.split('$')[1], 'c')+' '
            # prof principal
            if dico.has_key('ENTAuxEnsClassesPrincipal'):
                dico['ppal'] = ok_groupe(dico['ENTAuxEnsClassesPrincipal'].split('\n')[0].split('$')[1], 'c')
            else:
                dico['ppal'] = '0'
            # matières
            dico['matieres'] = ''
            if dico.has_key('ENTAuxEnsMatiereEnseignEtab'):
                for mat in dico['ENTAuxEnsMatiereEnseignEtab'].split('\n'):
                    dico['matieres'] += ok_groupe(mat.split('$')[1], 'm')+' '
            # disciplines
            if dico.has_key('ENTAuxEnsCategoDisciplines'):
                dico['disc'] = ok_groupe(dico['ENTAuxEnsCategoDisciplines'].split('\n')[0].split('$')[1], 'm')
            else:
                dico['disc'] = ''
            ## FIXME/TODO : utiliser le champ "mail" !
            chaine = '%(sn)s:%(givenName)s:%(sexe)s:%(ENTPersonJointure)s:%(ppal)s:%(date)s:%(matieres)s:%(classes)s:%(disc)s\n'%dico
            f_scribe.write(chaine)
        except:
            # entrée invalide
            print "Entrée invalide<br>"
            continue
        valide += 1
        if (valide != 0 and valide%50  == 0):
            print "<font size=-1>%d professeurs lus</font><br>"%valide
            sys.stdout.flush()

    ## FIXME : deleteRequest
    #print "Entrées supprimées"
    #for request in root_node.getElementsByTagName('deleteRequest'):
    #    for operational in request.getElementsByTagName('operationalAttributes'):
    #        attr = operational.getElementsByTagName('attr')[0]
    #        print attr.getAttribute("name"), ":", get_text_node(attr.getElementsByTagName('value')[0])
    #    for identifier in request.getElementsByTagName('identifier'):
    #        print "identifiant :", get_text_node(identifier.getElementsByTagName('id')[0])


if __name__ == "__main__":
    #enseigne = {}
    #profmat = {}
    valide = 0
    mode = 'w' # base unique
    scribe_file = "./tmp/temp-profs.txt"
    #gepi_file = "./tmp/prof_disc_classes.tmp"
    try:
        aaf_file = sys.argv[1]
        #mode = get_mode()
    except:
        print "Erreur lors de l'acquisition des arguments"
        sys.exit()
    f_scribe = open_file(scribe_file, mode)
    #f_gepi = open_file(gepi_file, mode)
    f_xml = open_file(aaf_file, 'r')
    f_xml.close()
    parse_xml_file(aaf_file)
    f_scribe.close()
    #f_gepi.close()
    print "<h3>%d professeurs ont été extraits</h3>"%valide

