#!/usr/bin/env python
# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# scp2scribe.py
#
# Parseur DOM pour fichiers XML issus de Sconet
#
###########################################################################

import xml.dom.minidom
import sys


def get_text_node(node):
    """
    @param node: node minidom contenant du texte
    Utilitaire minidom permettant de récupérer le texte d'un node texte
    """
    texte = ""
    nodelist=node.childNodes
    for textnode in nodelist:
        if textnode.nodeType == textnode.TEXT_NODE:
            texte = texte + textnode.data
    return texte


#def enregistre(enseignant, classe, matiere):
#    """
#    sauvegardes des affectations des professeurs
#    """
#    global enseigne, profmat
#    if enseigne.has_key(enseignant):
#        if classe not in enseigne[enseignant]:
#            enseigne[enseignant].append(classe)
#        if matiere not in profmat[enseignant]:
#            profmat[enseignant].append(matiere)
#    else:
#        enseigne[enseignant] = [classe]
#        profmat[enseignant] = [matiere]


def ok_groupe(groupe, gtype='g'):
    """
    portage de la fonction PHP originale
    """
    if gtype == 'n':
        # niveau coupé au 1er espace
        groupe = groupe.split()[0]
    groupe = groupe.split('(')[0]     #parentheses
    groupe = groupe.replace(' ', '')  #espaces
    groupe = groupe.replace('*', 'e') #etoile, ex : mp*
    groupe = groupe.replace('.', '')  #point
    groupe = groupe.replace(':', '')  #2points
    # groupes 100% numériques
    if groupe.isdigit():
        # ajout du prefixe "type"
        groupe = gtype+groupe
    return groupe
    

def get_mode():
    """
    gestion du mode multi-bases
    """
    if sys.argv[2] == '1':
        return "w+"
    return "a"


def open_file(filename, mode):
    try:
        return file(filename, mode)
    except:
        print "Impossible d'ouvrir le fichier %s"%filename
        sys.exit()

    
def parse_xml_file(filename):
    """
    parsing du fichier XML de STS-Web
    """
    # FIXME : ajouter des tests d'erreur :)
    document = xml.dom.minidom.parse(filename)
    root_node = document.documentElement
    
    ## recuperation des affectations des classes et des groupes
    division,groupes = parse_structures(root_node.getElementsByTagName('STRUCTURES')[0])
    ## recuperation des adresses renseignées
    adresses = parse_adresses(root_node.getElementsByTagName('ADRESSES')[0])
    ## recuperation des eleves
    parse_eleves(root_node.getElementsByTagName('ELEVES')[0], division,groupes,adresses)
    

def parse_structures(structures):
    """
    récupération des affectation des élèves aux matière et aux groupes
    """
    division = {}
    groupes = {}
    for eleve in structures.getElementsByTagName('STRUCTURES_ELEVE'):
        elenoet = eleve.getAttribute('ELENOET')
        groupes[elenoet] = []
        for struct in eleve.getElementsByTagName('STRUCTURE'):
            nom = get_text_node(struct.getElementsByTagName('CODE_STRUCTURE')[0])
            tip = get_text_node(struct.getElementsByTagName('TYPE_STRUCTURE')[0])
            if tip == 'D':
                division[elenoet] = ok_groupe(nom, 'c')
            else:
                groupes[elenoet].append(ok_groupe(nom, 'g'))
    return division,groupes


def parse_adresses(adresses):
    """
    récupération des adresses renseignees
    """
    adresse = {}
    for ad in adresses.getElementsByTagName('ADRESSE'):
        adid = ad.getAttribute('ADRESSE_ID')
        string = ''
        for champ in ['LIGNE1_ADRESSE', 'LIGNE2_ADRESSE', 'LIGNE3_ADRESSE', 'LIGNE4_ADRESSE', 'CODE_POSTAL', 'LIBELLE_POSTAL', 'LL_PAYS' ]:
            try:
                ligne = get_text_node(ad.getElementsByTagName(champ)[0])
                if ligne != '':
                    string += ligne+' '
            except:
                pass
        if string != '':
            adresse[adid] = string.replace(':','')
    return adresse 


def parse_eleves(eleves, division, groupes, adresses):
    global valide, f_scribe
    for eleve in eleves.getElementsByTagName('ELEVE'):
        elenoet = eleve.getAttribute('ELENOET')
        tab = {'ELENOET':elenoet}
        try:
            tab['DVISION'] = division[elenoet]
        except KeyError:
            # FIXME : eleve non affecté : on le saute !
            continue
        for champ in ['NOM', 'PRENOM', 'DATE_NAISS', 'CODE_SEXE' ]:
            try:
                tab[champ] = get_text_node(eleve.getElementsByTagName(champ)[0])
            except IndexError:
                # FIXME : en général, c'est CODE_SEXE qui manque !
                print champ, "non défini pour", elenoet
                tab[champ] = ''
        try:
            num = get_text_node(eleve.getElementsByTagName('ADRESSE_ID')[0])            
            tab['ADRESSE'] = adresses[num]
        except:
            tab['ADRESSE'] = "adresse non communiquee"

        print "%(ELENOET)s:%(NOM)s:%(PRENOM)s:%(CODE_SEXE)s:NIVEAU:%(DVISION)s:%(ADRESSE)s::"%tab
        valide += 1
        #if (valide != 0 and valide%100  == 0):
        #    print "<font size=-1>%d élèves lus</font><br>"%valide
    
    
def parse_individus(individus):
    global enseigne, profmat, valide, f_scribe
    if enseigne == {}:
        print "<font color=red>Les équipes pédagogiques n'ont pas été renseignées dans STS !</font><br>"
    for individu in individus.getElementsByTagName('INDIVIDU'):
        tab = {}
        tab['ID'] = individu.getAttribute('ID')
        # gestion de la balise prof principal
        pp = individu.getElementsByTagName('PROF_PRINC')
        if pp != []:
            classe = get_text_node(pp[0].getElementsByTagName('CODE_STRUCTURE')[0])            
            tab['PRINC'] = ok_groupe(classe, 'c')
        else:
            tab['PRINC'] = '0'
        # gestion des disciplines : une seule ?
        di = individu.getElementsByTagName('DISCIPLINE')
        if di != []:
            discipline = get_text_node(di[0].getElementsByTagName('LIBELLE_COURT')[0])
            tab['DISC'] = ok_groupe(discipline, 'm')
        else:
            tab['DISC'] = ''         
        # autres champs
        for champ in ['NOM_USAGE','PRENOM','DATE_NAISSANCE','SEXE']:
            tab[champ] = get_text_node(individu.getElementsByTagName(champ)[0])
        if tab['SEXE'] == '2' :
            tab['SEXE'] = '0'
        tab['DATE_NAISSANCE'] = tab['DATE_NAISSANCE'].replace('-','')
        if enseigne.has_key(tab['ID']):
            tab['ENSEIGNE'] = ' '.join(enseigne[tab['ID']])
        else:
            tab['ENSEIGNE'] = ''
        if profmat.has_key(tab['ID']):
            tab['MAT'] = ' '.join(profmat[tab['ID']])
        else:
            tab['MAT'] = ''
        f_scribe.write("%(NOM_USAGE)s:%(PRENOM)s:%(SEXE)s:%(ID)s:%(PRINC)s:%(DATE_NAISSANCE)s:%(MAT)s:%(ENSEIGNE)s:%(DISC)s\n" % tab)
        valide += 1
        if (valide != 0 and valide%50  == 0):
            print "<font size=-1>%d professeurs lus</font><br>"%valide
    

if __name__ == "__main__":
    #enseigne = {}
    #profmat = {}
    valide = 0
    scribe_file = "./tmp/temp-eleves.txt"
    gepi_file = './tmp/eleves_options.tmp'
    try:
        sco_file = sys.argv[1]
        mode = get_mode()
    except:
        print "Erreur lors de l'acquisition des arguments"
        sys.exit()
    f_scribe = open_file(scribe_file, mode)
    f_gepi = open_file(gepi_file, mode)
    f_xml = open_file(sco_file, 'r')
    f_xml.close()
    parse_xml_file(sco_file)
    f_scribe.close()
    f_gepi.close()
    print "<h3>%d élèves ont été extraits</h3>"%valide

