#!/usr/bin/env python
# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# sts2scribe.py
#
# Parseur DOM pour fichiers XML issus de STS-Web
#
###########################################################################

import xml.dom.minidom
import sys


def get_text_node(node):
    """
    @param node: node minidom contenant du texte
    Utilitaire minidom permettant de récupérer le texte d'un node texte
    """
    texte = ""
    nodelist=node.childNodes
    for textnode in nodelist:
        if textnode.nodeType == textnode.TEXT_NODE:
            texte = texte + textnode.data
    return texte


def enregistre(enseignant, classe, matiere):
    """
    sauvegardes des affectations des professeurs
    """
    global enseigne, profmat
    if enseigne.has_key(enseignant):
        if classe not in enseigne[enseignant]:
            enseigne[enseignant].append(classe)
        if matiere not in profmat[enseignant]:
            profmat[enseignant].append(matiere)
    else:
        enseigne[enseignant] = [classe]
        profmat[enseignant] = [matiere]


def ok_groupe(groupe, gtype='g'):
    """
    portage de la fonction PHP originale
    """
    if gtype == 'n':
        # niveau coupé au 1er espace
        groupe = groupe.split()[0]
    groupe = groupe.split('(')[0]     #parentheses
    groupe = groupe.replace(' ', '')  #espaces
    groupe = groupe.replace('*', 'e') #etoile, ex : mp*
    groupe = groupe.replace('.', '')  #point
    groupe = groupe.replace(':', '')  #2points
    groupe = groupe.replace('=', '')  #égal
    groupe = groupe.replace('"', '')  #double-quotes
    # groupes 100% numériques
    if groupe.isdigit():
        # ajout du prefixe "type"
        groupe = gtype+groupe
    return groupe
    

def get_mode():
    """
    gestion du mode multi-bases
    """
    if sys.argv[2] == '1':
        return "w+"
    return "a"


def open_file(filename, mode):
    try:
        return file(filename, mode)
    except:
        print "Impossible d'ouvrir le fichier %s"%filename
        sys.exit()

    
def parse_xml_file(filename):
    """
    parsing du fichier XML de STS-Web
    """
    # FIXME : ajouter des tests d'erreur :)
    document = xml.dom.minidom.parse(filename)
    root_node = document.documentElement
    # recuperation des matieres
    code_matiere = parse_nomenclatures(root_node.getElementsByTagName('NOMENCLATURES')[0])
    if code_matiere == {}:
        print "<font color=red>Aucune matière n'a été définie !</font><br>";
    # affectation des classes
    parse_divisions(root_node.getElementsByTagName('DIVISIONS')[0], code_matiere)
    # affectation des groupes
    parse_groupes(root_node.getElementsByTagName('GROUPES')[0], code_matiere)
    # recuperation des enseigants
    parse_individus(root_node.getElementsByTagName('INDIVIDUS')[0])
    

def parse_nomenclatures(nomenclatures):
    """
    récupération des MATIERES
    renvoit un dictionnaire : code->nom
    """
    code_matiere = {}
    for mat in nomenclatures.getElementsByTagName('MATIERE'):
        code_mat = mat.getAttribute('CODE')
        code_gestion = mat.getElementsByTagName('CODE_GESTION')[0]
        code_matiere[code_mat] = get_text_node(code_gestion)
    return code_matiere


def parse_divisions(divisions, code_matiere):
    """
    récupération des DIVISIONS avec MATIERES et ENSEIGNANTS
    """
    global f_gepi
    for division in divisions.getElementsByTagName('DIVISION'):
        classe = division.getAttribute('CODE')
        classe = ok_groupe(classe, 'c')
        for service in division.getElementsByTagName('SERVICE'):
            type_cours = service.getAttribute('CODE_MOD_COURS')
            matiere = code_matiere[service.getAttribute('CODE_MATIERE')]
            matiere = ok_groupe(matiere, 'm')
            for enseignant in service.getElementsByTagName('ENSEIGNANT'):
                ens = enseignant.getAttribute('ID')
                enregistre(ens, classe, matiere)
                if type_cours == 'CG':
                    f_gepi.write("%s#%s;%s;%s\n" % (ens, matiere.lower(), classe.lower(), type_cours))
                

def parse_groupes(groupes, code_matiere):
    """
    récupération des GROUPES avec ENSEIGNANTS et CLASSE
    """
    global f_gepi
    for groupe in groupes.getElementsByTagName('GROUPE'):
        appartenance = []
        for division in groupe.getElementsByTagName('DIVISION_APPARTENANCE'):
            classe = ok_groupe(division.getAttribute('CODE'), 'c')            
            appartenance.append(classe)
        for service in groupe.getElementsByTagName('SERVICE'):
            matiere = code_matiere[service.getAttribute('CODE_MATIERE')]
            matiere = ok_groupe(matiere, 'm')
            for enseignant in service.getElementsByTagName('ENSEIGNANT'):
                ens = enseignant.getAttribute('ID')
                for classe in appartenance:
                    enregistre(ens, classe, matiere)
                    f_gepi.write("%s#%s;%s;%s\n" % (ens, matiere.lower(), classe.lower(), 'OPT'))
                

def parse_individus(individus):
    global enseigne, profmat, valide, f_scribe
    if enseigne == {}:
        print "<font color=red>Les équipes pédagogiques n'ont pas été renseignées dans STS !</font><br>"
    for individu in individus.getElementsByTagName('INDIVIDU'):
        tab = {}
        tab['ID'] = individu.getAttribute('ID')
        # gestion de la balise prof principal
        pp = individu.getElementsByTagName('PROF_PRINC')
        if pp != []:
            classe = get_text_node(pp[0].getElementsByTagName('CODE_STRUCTURE')[0])            
            tab['PRINC'] = ok_groupe(classe, 'c')
        else:
            tab['PRINC'] = '0'
        # gestion des disciplines : une seule ?
        di = individu.getElementsByTagName('DISCIPLINE')
        if di != []:
            discipline = get_text_node(di[0].getElementsByTagName('LIBELLE_COURT')[0])
            tab['DISC'] = ok_groupe(discipline, 'm')
        else:
            tab['DISC'] = ''         
        # autres champs
        for champ in ['NOM_USAGE','PRENOM','DATE_NAISSANCE','CIVILITE']:
            try:
                tab[champ] = get_text_node(individu.getElementsByTagName(champ)[0])
            except:
                tab[champ] = ''
        if tab['CIVILITE'] == '' :
            # mec par défaut :)
            tab['CIVILITE'] = '1'
        tab['DATE_NAISSANCE'] = tab['DATE_NAISSANCE'].replace('-','')
        if enseigne.has_key(tab['ID']):
            tab['ENSEIGNE'] = ' '.join(enseigne[tab['ID']])
        else:
            tab['ENSEIGNE'] = ''
        if profmat.has_key(tab['ID']):
            tab['MAT'] = ' '.join(profmat[tab['ID']])
        else:
            tab['MAT'] = ''
        f_scribe.write("%(NOM_USAGE)s:%(PRENOM)s:%(CIVILITE)s:%(ID)s:%(PRINC)s:%(DATE_NAISSANCE)s:%(MAT)s:%(ENSEIGNE)s:%(DISC)s\n" % tab)
        valide += 1
        if (valide != 0 and valide%50  == 0):
            print "<font size=-1>%d professeurs lus</font><br>"%valide
    

if __name__ == "__main__":
    enseigne = {}
    profmat = {}
    valide = 0
    scribe_file = "./tmp/temp-profs.txt"
    gepi_file = "./tmp/prof_disc_classes.tmp"
    try:
        sts_file = sys.argv[1]
        mode = get_mode()
    except:
        print "Erreur lors de l'acquisition des arguments"
        sys.exit()
    f_scribe = open_file(scribe_file, mode)
    f_gepi = open_file(gepi_file, mode)
    f_xml = open_file(sts_file, 'r')
    f_xml.close()
    parse_xml_file(sts_file)
    f_scribe.close()
    f_gepi.close()
    print "<h3>%d professeurs ont été extraits</h3>"%valide

